@@ -336,6 +336,10 @@ gboolean    gjs_unichar_from_string          (JSContext       *context,
 
 const char* gjs_get_type_name                (jsval            value);
 
+JSBool      gjs_value_to_int64               (JSContext       *context,
+                                              const jsval      val,
+                                              gint64          *result);
+
 jsval       gjs_date_from_time_t             (JSContext *context, time_t time);
 
 JSBool      gjs_parse_args                   (JSContext  *context,
